#
# New plot.r for use with PGPLOT library.
# The help files for these functions are in
# misc/plhelp
#

# pgplot.r

# This file is a part of RLaB ("Our"-LaB)
# Copyright (C) 1997  Ian R. Searle

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# See the file ../COPYING

#
# If your system does not deal with Infs and NaNs well, then
# uncomment the following lines.
#

# static (isinf, isnan)
#
# isinf = function ( A ) { return (0); };
# isnan = function ( A ) { return (0); };

static (WIN)		# The static plot window structure
static (P)		# The active/current plot window

#
# Maintain the transformations for 3-D plots.
#

static (basex, basey, height)
basex = 2;
basey = 2;
height = 4;

#
# Static (private) functions. For use from within
# this file only.
#

static (create_plot_object)
static (check_plot_object)
static (xy_scales)
static (x_scales)
static (y_scales)
static (z_scales)
static (XYZ_scales)
static (list_scales)
static (list_sort)
static (hist_scales)
static (plot_matrix)
static (plot_list)
static (check_3d_list)
static (find_char)
static (get_style)
static (make_legend)
static (plhold_first)

#
# Defaults
#

static (grid_x_default, grid_y_default)
static (grid_3x_default, grid_3y_default, grid_3z_default)

grid_x_default = "bcnst";
grid_y_default = "bcnstv";
grid_3x_default = "bnstu";
grid_3y_default = "bnstu";
grid_3z_default = "bcdmnstuv";

static (subplot_spec)          # Pay attention to the specifications ?
static (subplot_horizontal)    # The horizontal specification.
static (subplot_vertical)      # The vertical specification.

subplot_spec = 0;

#
# Initial windows size/config...
#

static (win_xp_df, win_yp_df)
static (win_xleng_df, win_yleng_df)
static (win_xoff_df, win_yoff_df)

win_xp_df = 0;
win_yp_df = 0;
win_xleng_df = 400;
win_yleng_df = 300;
win_xoff_df = 200;
win_yoff_df = 200;

#
# Create the default plot-object.
# Initialize to all the default values
#

if (!exist (WIN)) 
{
  # Create the plot-object list
  WIN = <<>>;
}

create_plot_object = function ( N, nx, ny )
{
  if (!exist (N)) { N = 1; }
  
  pobj = <<>>;

  pobj.window.xp = 0;           # Number of X pixels
  pobj.window.yp = 0;           # Number of Y pixels
  pobj.window.xleng = 400;      # Page length, X
  pobj.window.yleng = 300;      # Page length, Y
  pobj.window.xoff = 200;       # Page offset, X
  pobj.window.yoff = 200;       # Page offset, Y

  pobj.subplot = 0;		# The current subplot no.
  pobj.nplot = nx*ny;		# Total no. of plots on window
  pobj.nx = nx;
  pobj.ny = ny;

  pobj.fontld = 0;		# Loaded extended fonts?

  pobj.char_height = 2.0;       # Character height.
  
  for (i in 1:(nx*ny))
  {
    pobj.style.[i] = "line";         # The type/style of plot to draw
    pobj.nbin[i] = 1j;          # The number of bins for a histogram
    pobj.font[i] = 1;		# The current font
    pobj.height[i] = 1;		# The character height
    pobj.xlabel[i] = "";
    pobj.ylabel[i] = "";
    pobj.zlabel[i] = "";
    pobj.title[i] = "";
    pobj.orientation[i] = "portrait";
    pobj.desc.[i] = "default";		# The legend description
    pobj.gridx[i] =  grid_x_default;	# Plot axes style, 2D-X
    pobj.gridy[i] =  grid_y_default;	# Plot axes style, 2D-Y
    pobj.grid3x[i] = grid_3x_default;	# Plot axes style, 3D-X
    pobj.grid3y[i] = grid_3y_default;	# Plot axes style, 3D-Y
    pobj.grid3z[i] = grid_3z_default;	# Plot axes style, 3D-Z
    pobj.aspect[i] = 0;		        # Plot aspect style
    pobj.alt[i] = 60;
    pobj.az[i] = 45;
    
    pobj.xmin[i] = 1j;
    pobj.xmax[i] = 1j;
    pobj.ymin[i] = 1j;
    pobj.ymax[i] = 1j;
    pobj.zmin[i] = 1j;
    pobj.zmax[i] = 1j;
    
    pobj.page.xp = 0;
    pobj.page.yp = 0;
    pobj.xleng = 400;
    pobj.yleng = 300;
    pobj.xoff = 200;
    pobj.yoff = 200;

    pobj.color[i;] = 1:14;               # 14 possible colors...
    pobj.lstyle[i;] = [1:5,1:5,1:5,1:5]; # 5 possible line styles...
    pobj.pstyle[i;] = 1:8;               # 8 possible point styles...
    pobj.width[i;] = ones(1,32);         # 32 possible point line widths
  }
  
  #
  # Save the newly generated plot-object
  # in a list of plot-objects.
  
  WIN.[N] = pobj;
};

##############################################################################
#
# (Re) Set the initial plot window specs...

plwin_init = function (xp, yp, xleng, yleng, xoff, yoff)
{
  if (!exist (xp)) 
  { 
    win_xp_df = 0;
  else
    win_xp_df = xp;
  }

  if (!exist (yp)) 
  { 
    win_yp_df = 0;
  else
    win_yp_df = yp;
  }

  if (!exist (xleng)) 
  { 
    win_xleng_df = 0;
  else
    win_xleng_df = xleng;
  }

  if (!exist (yleng)) 
  { 
    win_yleng_df = 0;
  else
    win_yleng_df = yleng;
  }

  if (!exist (xoff)) 
  { 
    win_xoff_df = 0;
  else
    win_xoff_df = xoff;
  }

  if (!exist (yoff)) 
  { 
    win_yoff_df = 0;
  else
    win_yoff_df = yoff;
  }
};

##############################################################################
#
# Check to make sure a plot-object exists. If one
# does not exist, create it.
#

check_plot_object = function ()
{
  if (length (WIN) == 0)
  {
    plstart();
    return 0;
  }
  return 1;
};

##############################################################################
#
# Show the current plot-window, and the possibilities
#

showplwin = function ( all )
{
  if (length (WIN) == 0)
  {
    printf ("No plot objects\n");
    return 0;
  }
  
  printf ("Current plot-window is:\t\t%i\n", P);
  printf ("Available plot windows are:\t");
  for (i in members (WIN))
  {
    printf ("%s   ", i);
  }
  printf ("\n");

  if (exist (all))
  {
    for (i in members (WIN.[P]))
    {
      WIN.[P].[i]
    }
  }
};

getplot = function ( win_no )
{
  local (win_no)

  if (length (WIN) != 0)
  {
    if (!exist (win_no)) { win_no = P; }

    if (exist (WIN.[win_no]))
    {
      return (WIN.[win_no]);
    else
      return 0;
    }
  }
  return <<>>;
};

##############################################################################
#
# Set/start/select the plot device
#
# plstart ( NX, NY, DEV, WIDTH, ASPECT )
#
# Plstart opens a plot-device (window) and prepares it for
# plotting. Plstart returns the window ID (a scalar). NX and NY
# specify the number of plots in the horizontal and vertical
# directions respectively. DEV specifies the plot-window device. Valid
# examples are "/XWIN", "/PS", "/CPS". WIDTH specifies the width of
# the plot window in inches, while ASPECT specifies the aspect ratio
# of the new plot window.  Defaults for each argument are:
#
#   NX:     1
#   NY:     1
#   DEV:    "/XWIN"
#   WIDTH:  (user's .Xdefaults or environment)
#   ASPECT: 1 (only if WIDTH is specified).
#

plstart = function ( nx, ny, dev, width, aspect )
{
  if (!exist (nx)) { nx = 1; }
  if (!exist (ny)) { ny = 1; }
  if (!exist (dev)) { dev = "/XWIN"; }

  # Create the plot-object
  # First, figure out the index
  if (!exist (P))
  {
    P = 1;
  else
    P = P + 1;
  }
  
  create_plot_object (P, nx, ny);

  # Open the plot device.
  winid = pgopen (dev);

  # Setup the window width/aspect ratio.
  if (exist (width))
  {
    if (!exist (aspect)) { aspect = 1; }
    pgpap (width, aspect);
  }

  # Turn off page prompting.
  pgask (0);

  # Set up the number of plots per window.
  pgsubp (nx, ny);

  return winid;
};

##############################################################################
#
# Close a plot device. We must destroy the current plot-object
# And switch the output stream back to the default.
#

plclose = function ()
{
  if (size (WIN) > 1)
  {   
    #
    # Clear WIN.[P] and reset P to 1st plot-window
    #

    clear (WIN.[P]);
    pgclos ();
    pgslct (strtod (members (WIN)[1]));
    P = strtod (members (WIN)[1]);
    return P;

  else if (size (WIN) == 1) {

    if (exist (WIN.[P])) 
    { 
      clear (WIN.[P]); 
      clear (P);
    }
    pgclos ()
    return 1;

  else if (size (WIN) == 0) {

    return 0;

  } } }
};

##############################################################################
#
# Close ALL the plot-windows
#

plend = function ()
{
  pgend ();
  if (exist (WIN)) { clear (WIN); }
  if (exist (P)) { clear (P); }
  WIN = <<>>;
};

##############################################################################
#
# Change plot aspect ratio
#

plaspect = function ( aspect )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  if (!exist (aspect))
  {
    WIN.[P].aspect[i] = 0;
  else
    WIN.[P].aspect[i] = aspect;
  }
};

##############################################################################
#
# Set the current suplot to draw to.
#
# subplot ( HORIZONTAL, VERTICAL )
#
# HORIZONTAL specifies the subpanel in the horizontal direction
# (columns) to use for the next plot. VERTICAL specifies the subpanel
# in the vertical direction (rows) to use for the next plot.
#
# Both HORIZONTAL and VERTICAL must be specified for subplot to have
# any effect at all. Otherwise, the next sub-panel is used for the
# next plot.
#
# subplot must be used to specifiy the placement of each and every
# plot whose placement differs from the normal sub-panel advancement
# (horizontal, then vertical).
#

subplot = function ( HORIZONTAL, VERTICAL )
{
  if (!exist (HORIZONTAL))
  {
    subplot_spec = 0;
    return 0;
  }

  if (!exist (VERTICAL))
  {
    subplot_spec = 0;
    return 0;
  }

  # Turn on the subplot indicator.
  subplot_spec = 1;

  # Set the subplot to use next.
  subplot_horizontal = HORIZONTAL;
  subplot_vertical = VERTICAL;

  return [subplot_horizontal, subplot_vertical];
};

##############################################################################
#
# Change plot line style
#

plstyle = function ( style )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;

  if (exist (style)) 
  {
    if (class (style) == "string") 
    {
      for (j in 1:style.n)
      {
	if (style[j] != "line" && ...
            style[j] != "point" && ...
            style[j] != "line-point")
	{
	  error ("plstyle: STYLE must be either " + ...
	         "\"point\", \"line\" or \"line-point\"");
	}
      }
      WIN.[P].style.[i] = style;
    }
    return 1;
  }
  WIN.[P].style.[i] = "line";
  return 1;
};

##############################################################################
#
# Control of the plot line style.
# There are 5 line styles

plline = function ( line_style )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;

  if (exist (line_style)) 
  {
    if (class (line_style) == "num") 
    {
      if (line_style.n != 20)
      {
	error ("plline: LVEC must be 1x20 in size");
      }
      WIN.[P].lstyle[i;] = line_style;
    }
    return 1;
  }
  WIN.[P].lstyle[i;] = [1:5,1:5,1:5,1:5];
  return 1;
};

##############################################################################
#
# Control of the plot line color
#

plcolor = function ( COLOR )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;

  if (exist (COLOR)) 
  {
    if (class (COLOR) == "num") 
    {
      WIN.[P].color[i;] = COLOR;
    }
    return 1;
  }
  WIN.[P].COLOR[i;] = 1:14;
  return 1;
};

##############################################################################
#
# Control of the plot point style.
# There are 8 line styles

plpoint = function ( point_style )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;

  if (exist (point_style)) 
  {
    if (class (point_style) == "num") 
    {
      if (point_style.nc != 8)
      {
	error ("plpoint: PVEC must be 1x8 in size");
      }
      WIN.[P].pstyle[i;] = point_style;
    }
    return 1;
  }
  WIN.[P].pstyle[i;] = 1:8;
  return 1;
};

##############################################################################
#
# Get the right value of line-style
#

get_style = function ( STY, K )
{
  local (sty);
  sty = mod(K, STY.n);
  if(sty == 0) 
  { 
    sty = STY.n; 
  }
  return STY[sty];
};

##############################################################################
#
# Change fonts
#

plfont = function ( font )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  
  if (!exist (font)) { font = 1; }
  
  if (WIN.[P].fontld == 0)
  {
    pgscf (1);
    WIN.[P].fontld = 1;
  }
  
  WIN.[P].font[i] = font;
  return P;
};

##############################################################################
#
# Change fonts
#

plch = function ( H )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  
  if (!exist (H)) { H = 1; }
  
  WIN.[P].char_height = H;
  
  return P;
};

##############################################################################
#
# Change pen width
#

plwid = function ( width )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  
  if (!exist (width)) 
  { 
    WIN.[P].width = ones(1,32);
  else
    WIN.[P].width[i] = width;
  }
  return P;
};

##############################################################################
#
# Place some text on the plot
#

plptex = function ( text, x , y , angle , just )
{
  if (!check_plot_object ()) 
  {
    printf ("Must use plot() before plptex()\n");
    return 0;
  }
  
  if (!exist (x)) { x = 0; }
  if (!exist (y)) { y = 0; }
  if (!exist (angle)) { angle = 0; }
  if (!exist (just)) { just = 0; }
  
  pgptxt (x, y, angle, just, text);
  pgupdt ();
};

##############################################################################
#
# Find a character in a string
#

find_char = function ( str , char )
{  
  tmp = strsplt (str);
  for (i in 1:tmp.n)
  {
    if (tmp[i] == char) 
    {
      return i;
    }
  }
  return 0;
};

##############################################################################
#
# Sort list element names/labels by numeric order, then string order.
#

list_sort = function ( L )
{
  tl = <<>>;
  j = k = 1;

  for (i in members (L))
  {
    if (!isnan (strtod (i)))
    {
      num[j] = i;
      j++;
      else
      char[k] = i;
      k++;
    }
  }
  
  # Sort the numeric labels
  
  if (exist (num))
  {
    num = sort (strtod (num)).val;
    tl.num = num;
  }

  if (exist (char))
  {
    tl.char = char;
  }

  return tl;
};

##############################################################################
#
# Plot the columns of a matrix (X-Y plot).
#
##############################################################################

plot = function ( data, key, textf )
{
  check_plot_object ();
  
  p = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;	# The current index
  if (!exist (key)) { key = 1; }
  if (!exist (textf)) { textf = 1; }

  #
  # Draw the graph
  # Step through the matrix plotting
  # each column versus the 1st
  #
  
  if (subplot_spec)
  {
    subplot_horizontal
    pgpanl (subplot_horizontal, subplot_vertical);
    subplot_spec = 0;
  else
    pgpage ();
  }

  if (class (data) == "num")
  {
    #
    # Set up the plot basics
    #

    if (abs (key) > data.nc)
    {
      error ("plot: KEY argument > M.nc");
    }
    
    pgsci (1);                        # Set color
    pgsls (1);                        # Set line style
    pgslw (1);
    pgscf (WIN.[P].font[p]);          # Set font.
    pgsch (WIN.[P].char_height);      # Set character height
    
    #
    # Compute scale limits
    #

    k = find ((1:data.nc) != abs (key));
    if (key > 0)
    {
      if (data.nc != 1)
      {
	</xmax;xmin/> = x_scales (real(data)[;key], p);
	</ymax;ymin/> = y_scales (real(data)[;k],   p);
      else
	</xmax;xmin/> = x_scales ((1:data.nr)', p);
	</ymax;ymin/> = y_scales (real(data),   p);
      }
    else if (key < 0) {
      </xmax;xmin/> = x_scales (real(data)[;k],   p);
      </ymax;ymin/> = y_scales (real(data)[;abs(key)], p);
    else
      </xmax;xmin/> = x_scales ((1:data.nr)', p);
      </ymax;ymin/> = y_scales (real(data),   p);
    } }

    #
    # Set up the window/viewport.
    #

    pgvstd ();

    if (WIN.[P].aspect[p] != 0)
    {
      pgwnad (xmin, xmax, ymin, ymax);
    else
      pgswin (xmin, xmax, ymin, ymax);
    }

    pgbox (WIN.[P].gridx[p], 0, 0, WIN.[P].gridy[p], 0, 0);

    if (plot_matrix ( data, key, p, 0, xmin, xmax, ymin, ymax, ymax-ymin ) < 0) 
    {
      return -1;
    }
    
    else if (class (data) == "list") {
      
      pgsci (1);
      pgsls (1);
      pgslw (1);                        # Line width
      pgscf (WIN.[P].font[p]);          # Set character font
      pgsch (WIN.[P].char_height);      # Set character height
    

      </xmax;xmin;ymax;ymin/> = list_scales ( data, key, p );

      #
      # Set up the window/viewport.
      #
      
      pgvstd ();

      if (WIN.[P].aspect[p] != 0)
      {
	pgwnad (xmin, xmax, ymin, ymax);
      else
	pgswin (xmin, xmax, ymin, ymax);
      }

      pgbox (WIN.[P].gridx[p], 0, 0, WIN.[P].gridy[p], 0, 0);
      
      if (plot_list ( data, key, p, xmin, xmax, ymin, ymax ) < 0) 
      { 
	return -1;
      }

    else
      error ("plot: un-acceptable argument");
  } }

  pgsls (1);
  pgsci (1);

  pglab(WIN.[P].xlabel[p], WIN.[P].ylabel[p], WIN.[P].title[p]);  
  pgupdt ();

  #
  # Increment the plot no. so that next time
  # we use the correct settings.
  #
  
  WIN.[P].subplot = WIN.[P].subplot + 1;
  return P;
};

##############################################################################
#
# Plot the contours of a 2D data set.
#
# The data is composed in a list, with
# elements `x', `y', and `z'. x and y are single-dimension arrays
# (row or column matrices), and z is a two-dimensional array. The
# array z, is a function of x and y: z = f(x,y). Thus, the values in
# the array x can be thought of a "row-labels", and the values of y
# can be thought of as "column-lables" for the 2-dimensioal array z.
#
##############################################################################

plcont = function ( CL )
{
  check_plot_object ();
  
  #
  # 1st check list contents
  #
  
  if (exist (CL)) { check_3d_list (CL); }

  p = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;	# The current index
  
  #
  # Figure out the scale limits. 
  # Needs improvement!
  #

  xmin = xmax = ymin = ymax = zmin = zmax = 0;
  if (exist (CL)) 
  {
    </Xmax;Xmin;Ymax;Ymin;Zmax;Zmin/> = XYZ_scales (CL.x, CL.y, CL.z, p);
    if (Xmin < xmin) { xmin = Xmin; } if (Xmax > xmax) { xmax = Xmax; }
    if (Ymin < ymin) { ymin = Ymin; } if (Ymax > ymax) { ymax = Ymax; }
    if (Zmin < zmin) { zmin = Zmin; } if (Zmax > zmax) { zmax = Zmax; }
  }

  if (subplot_spec)
  {
    subplot_horizontal
    pgpanl (subplot_horizontal, subplot_vertical);
    subplot_spec = 0;
  else
    pgpage ();
  }

  pgsci (1);
  pgslw (1);
  pgsch (WIN.[P].char_height);       # Set character height

  #
  # Set up the 1st viewport for drawing the plot.
  #

  pgvstd ();

  if (WIN.[P].aspect[p] != 0)
  {
    pgwnad (1, CL.x.n, 1, CL.y.n);
    else
    pgswin (1, CL.x.n, 1, CL.y.n);
  }

  pgbox (WIN.[P].gridx[p], 0, 0, WIN.[P].gridy[p], 0, 0);

  # Convert the data to log data if necessary.
  if (find_char (WIN.[P].gridx[p], "l"))
  { x = log10 (real (CL.x)); else x = real (CL.x); }
  if (find_char (WIN.[P].gridy[p], "l"))
  { y = log10 (real (CL.y)); else y = real (CL.y); }
  z = real (CL.z);
  
  if (exist (CL.clevel))
  {
    clevel = CL.clevel;
  else
    clevel = linspace(zmin, zmax, 10);
  }

  #
  # Draw the contours
  #

  pgcont (CL.z, CL.x.n, CL.y.n, 1, CL.x.n, 1, CL.y.n, ...
          clevel, clevel.n, [0,1,0,0,0,1]);

  #
  # Reset color and draw the labels.
  #

  pglab (WIN.[P].xlabel[p], WIN.[P].ylabel[p], WIN.[P].title[p]);  

  #
  # Increment the plot no. so that next time
  # we use the correct settings.
  #
  
  WIN.[P].subplot = WIN.[P].subplot + 1;
  
  return P;
};

##############################################################################
#
##############################################################################

plimag = function ( CL )
{
  check_plot_object ();
  
  #
  # 1st check list contents
  #
  
  if (exist (CL)) { check_3d_list (CL); }

  p = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;	# The current index
  
  #
  # Figure out the scale limits. 
  # Needs improvement!
  #

  xmin = xmax = ymin = ymax = zmin = zmax = 0;
  if (exist (CL)) 
  {
    </Xmax;Xmin;Ymax;Ymin;Zmax;Zmin/> = XYZ_scales (CL.x, CL.y, CL.z, p);
    if (Xmin < xmin) { xmin = Xmin; } if (Xmax > xmax) { xmax = Xmax; }
    if (Ymin < ymin) { ymin = Ymin; } if (Ymax > ymax) { ymax = Ymax; }
    if (Zmin < zmin) { zmin = Zmin; } if (Zmax > zmax) { zmax = Zmax; }
  }

  if (subplot_spec)
  {
    subplot_horizontal
    pgpanl (subplot_horizontal, subplot_vertical);
    subplot_spec = 0;
  else
    pgpage ();
  }

  pgsci (1);
  pgslw (1);
  pgsch (WIN.[P].char_height);       # Set character height

  #
  # Set up the 1st viewport for drawing the plot.
  #

  pgvstd ();

  if (WIN.[P].aspect[p] != 0)
  {
    pgwnad (1, CL.x.n, 1, CL.y.n);
    else
    pgswin (1, CL.x.n, 1, CL.y.n);
  }

  pgbox (WIN.[P].gridx[p], 0, 0, WIN.[P].gridy[p], 0, 0);

  # Convert the data to log data if necessary.
  if (find_char (WIN.[P].gridx[p], "l"))
  { x = log10 (real (CL.x)); else x = real (CL.x); }
  if (find_char (WIN.[P].gridy[p], "l"))
  { y = log10 (real (CL.y)); else y = real (CL.y); }
  z = real (CL.z);
  
  if (exist (CL.clevel))
  {
    clevel = CL.clevel;
  else
    clevel = linspace(zmin, zmax, 10);
  }

  #
  # Draw the contours
  #

  pgimag (CL.z, CL.x.n, CL.y.n, 1, CL.x.n, 1, CL.y.n, ...
          zmin, zmax, [0,1,0,0,0,1]);
#  pggray (CL.z, CL.x.n, CL.y.n, 1, CL.x.n, 1, CL.y.n, ...
#          zmin, zmax, [0,1,0,0,0,1]);

  #
  # Reset color and draw the labels.
  #

  pglab (WIN.[P].xlabel[p], WIN.[P].ylabel[p], WIN.[P].title[p]);  

  #
  # Increment the plot no. so that next time
  # we use the correct settings.
  #
  
  WIN.[P].subplot = WIN.[P].subplot + 1;
  
  return P;
};

##############################################################################
#
# Plot histogram of binned data.
#
##############################################################################

plbin = function ( data )
{
  check_plot_object ();
  
  p = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;	# The current index
  if (!exist (key)) { key = 1; }
  if (!exist (textf)) { textf = 1; }

  #
  # Draw the graph
  # Step through the matrix plotting
  # each column versus the 1st
  #
  
  if (subplot_spec)
  {
    subplot_horizontal
    pgpanl (subplot_horizontal, subplot_vertical);
    subplot_spec = 0;
  else
    pgpage ();
  }

  pgsch (WIN.[P].char_height);      # Set character height
  pgsci (1);                        # Set color
  pgsls (1);                        # Set line style
  pgslw (1);
  pgscf (WIN.[P].font[p]);          # Set font.
    
  #
  # Compute scale limits
  #
  
  k = find ((1:data.nc) != abs (key));
  if (key > 0)
  {
    if (data.nc != 1)
    {
      </xmax;xmin/> = x_scales (real(data)[;key], p);
      </ymax;ymin/> = y_scales (real(data)[;k],   p);
      else
      </xmax;xmin/> = x_scales ((1:data.nr)', p);
      </ymax;ymin/> = y_scales (real(data),   p);
    }
    else if (key < 0) {
      </xmax;xmin/> = x_scales (real(data)[;k],   p);
      </ymax;ymin/> = y_scales (real(data)[;abs(key)], p);
      else
      </xmax;xmin/> = x_scales ((1:data.nr)', p);
      </ymax;ymin/> = y_scales (real(data),   p);
  } }

  #
  # Set up the window/viewport.
  #
  
  pgvstd ();

  if (WIN.[P].aspect[p] != 0)
  {
    pgwnad (xmin, xmax, ymin, ymax);
    else
    pgswin (xmin, xmax, ymin, ymax);
  }
  
  pgbox (WIN.[P].gridx[p], 0, 0, WIN.[P].gridy[p], 0, 0);
  
  #
  # Check for log scales.
  #

  if (find_char (WIN.[P].gridx[p], "l"))
  { data[;1] = log10 (data[;1]); }
  if (find_char (WIN.[P].gridy[p], "l"))
  { data[;2] = log10 (data[;2]); }  

  pgsci (3);
  pgbin (data.nr,  data[;1], data[;2], 1);
  pgsls (1);
  pgsci (1);

  pglab(WIN.[P].xlabel[p], WIN.[P].ylabel[p], WIN.[P].title[p]);  
  pgupdt ();

  #
  # Increment the plot no. so that next time
  # we use the correct settings.
  #
  
  WIN.[P].subplot = WIN.[P].subplot + 1;
  return P;
};

##############################################################################
#
# Copy a plot-object
#
# ID, the plot window ID to copy.
# DEV, the type of output device to open.
#

plcopy = function ( ID, DEV )
{
  #
  # Make sure ID exists in WIN
  #

  if (!exist (WIN.[ID]))
  {
    error ("plcopy: must specify valid plot-object");
  }

  #
  # Get the new object ID
  #

  if (!exist (P))
  {
    P = 1;
  else
    P = P + 1;
  }

  #
  # Perform the copy.
  #

  WIN.[P] = WIN.[ID];

  # Open the plot device.
  winid = pgopen (DEV);

  # Turn off page prompting.
  pgask (0);  

  # Set up the number of plots per window.
  pgsubp (WIN.[ID].nx, WIN.[ID].ny);

};

##############################################################################
#
# Set the X-axis label
#

xlabel = function ( xstr )
{
  check_plot_object ();
  if (!exist (xstr)) { xstr = ""; }
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  WIN.[P].xlabel[i] = xstr;
};

##############################################################################
#
# Set the Y-axis label
#

ylabel = function ( xstr )
{
  check_plot_object ();
  if (!exist (xstr)) { xstr = ""; }
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  WIN.[P].ylabel[i] = xstr;
};

##############################################################################
#
# Set the Z-axis label
#

zlabel = function ( xstr )
{
  check_plot_object ();
  if (!exist (xstr)) { xstr = ""; }
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  WIN.[P].zlabel[i] = xstr;
};

##############################################################################
#
# Set the plot-title
#

pltitle = function ( xstr )
{
  check_plot_object ();
  if (!exist (xstr)) { xstr = ""; }
  p = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;	# The current index
  WIN.[P].title[p] = xstr;
};

##############################################################################
#
# Set the scale limits.
#

plimits = function ( xmin, xmax, ymin, ymax, zmin, zmax )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;

  if (exist (xmin)) 
  {
    WIN.[P].xmin[i] = xmin;
  else
    WIN.[P].xmin[i] = 1j;
  }
  if (exist (xmax)) 
  { 
    WIN.[P].xmax[i] = xmax;
  else
    WIN.[P].xmax[i] = 1j;
  }

  if (exist (ymin)) 
  {
    WIN.[P].ymin[i] = ymin;
  else
    WIN.[P].ymin[i] = 1j;
  }
  if (exist (ymax)) 
  {
    WIN.[P].ymax[i] = ymax;
  else
    WIN.[P].ymax[i] = 1j;
  }

  if (exist (zmin)) 
  {
    WIN.[P].zmin[i] = zmin;
  else
    WIN.[P].zmin[i] = 1j;
  }
  if (exist (zmax)) 
  {
    WIN.[P].zmax[i] = zmax;
  else
    WIN.[P].zmax[i] = 1j;
  }
};

##############################################################################
#
# Set 2-D grid styles. A not-so-friendly interface.
#

plgrid = function ( sty_x, sty_y )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;

  if (exist (sty_x)) 
  { 
    if (class (sty_x) == "string")
    {
      WIN.[P].gridx[i] = sty_x;
    else
      error ("plgrid: requires string argument GRID_STY_X");
    }
  else
    WIN.[P].gridx[i] = grid_x_default;
  }
  if (exist (sty_y)) 
  { 
    if (class (sty_y) == "string")
    {
      WIN.[P].gridy[i] = sty_y;
    else
      error ("plgrid: requires string argument GRID_STY_Y");
    }
  else
    WIN.[P].gridy[i] = grid_y_default;
  }
};

##############################################################################
#
# Set 3-D grid (axis) styles
#

plgrid3 = function ( sty_x, sty_y, sty_z )
{  
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  if (exist (sty_x)) 
  { 
    if (class (sty_x) == "string")
    {
      WIN.[P].grid3x[i] = sty_x;
    else
      error ("plgrid3: requires string argument GRID_STY_X");
    }
  else
    WIN.[P].grid3x[i] = grid_3x_default;
  }
  if (exist (sty_y)) 
  { 
    if (class (sty_y) == "string")
    {
      WIN.[P].grid3y[i] = sty_y;
    else
      error ("plgrid3: requires string argument GRID_STY_Y");
    }
  else
    WIN.[P].grid3y[i] = grid_3y_default;
  }
  if (exist (sty_z)) 
  { 
    if (class (sty_z) == "string")
    {
      WIN.[P].grid3z[i] = sty_z;
    else
      error ("plgrid3: requires string argument GRID_STY_Z");
    }
  else
    WIN.[P].grid3z[i] = grid_3z_default;
  }
};

##############################################################################
#
# A friendlier interface to changing 2-D grid/axis
# styles.
#

plaxis = function ( X_STR, Y_STR )
{
  check_plot_object ();
  i = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;
  
  if (exist (X_STR))
  {
    if (X_STR == "log") { WIN.[P].gridx[i] = "bcngstl"; }
  else
    WIN.[P].gridx[i] = grid_x_default;
  }
  
  if (exist (Y_STR))
  {
    if (Y_STR == "log") { WIN.[P].gridy[i] = "bcngstlv"; }
  else
    WIN.[P].gridy[i] = grid_y_default;
  }
  return P;
};

##############################################################################
#
# Various internal support functions. Eventually these will be static.
#
##############################################################################

#
# Find the X or Y scale limits .
# M can be a multi-column matrix, all columns
# will be used.
#

x_scales = function ( M, p )
{  
  #
  # 1st check for un-plottable data
  #
  
  if (any (any (isinf (M))))
  { error ("plot: cannot plot Infs"); }
  if (any (any (isnan (M))))
  { error ("plot: cannot plot NaNs"); }
  
  xmin = min (min (M));
  xmax = max (max (M));
  
  #
  # Check computed scale limits against user's
  #
  
  if (WIN.[P].xmin[p] != 1j) { xmin = real (WIN.[P].xmin[p]); }
  if (WIN.[P].xmax[p] != 1j) { xmax = real (WIN.[P].xmax[p]); }

  #
  # Check for potential errors
  #
  
  if (xmin == xmax) 
  { 
    # As good a guess as any
    xmin = xmin - 1;
    xmax = xmax + 1;
  }
  
  #
  # Finally, adjust if log-scales
  #
  
  if (find_char (WIN.[P].gridx[p], "l"))
  {
    if (xmin <= 0 || xmax <= 0) 
    { 
      error ("cannot plot log(x<=0)"); 
    }
    xmin = log10 (xmin);
    xmax = log10 (xmax);
  }
  
  return << xmin = xmin ; xmax = xmax >>;
};

y_scales = function ( M, p )
{
  
  #
  # 1st check for un-plottable data
  #
  
  if (any (any (isinf (M))))
  { error ("plot: cannot plot Infs"); }
  if (any (any (isnan (M))))
  { error ("plot: cannot plot NaNs"); }
  
  xmin = min (min (M));
  xmax = max (max (M));
  
  #
  # Check computed scale limits against user's
  #
  
  if (WIN.[P].ymin[p] != 1j) { xmin = real (WIN.[P].ymin[p]); }
  if (WIN.[P].ymax[p] != 1j) { xmax = real (WIN.[P].ymax[p]); }

  #
  # Check for potential errors
  #
  
  if (xmin == xmax) 
  { 
    # As good a guess as any
    xmin = xmin - 1;
    xmax = xmax + 1;
  }
  
  #
  # Finally, adjust if log-scales
  #
  
  if (find_char (WIN.[P].gridy[p], "l"))
  {
    if (xmin <= 0 || xmax <= 0) { error ("cannot plot log(y<=0)"); }
    xmin = log10 (xmin);
    xmax = log10 (xmax);
  }
  
  return << xmin = xmin ; xmax = xmax >>;
};

z_scales = function ( M, p, xmin, xmax )
{
  
  #
  # 1st check for un-plottable data
  #
  
  if (any (any (isinf (M))))
  { error ("plot: cannot plot Infs"); }
  if (any (any (isnan (M))))
  { error ("plot: cannot plot NaNs"); }
  
  xmin = min (min (M));
  xmax = max (max (M));
  
  #
  # Check computed scale limits against user's
  #
  
  if (WIN.[P].zmin[p] != 1j) { xmin = real (WIN.[P].zmin[p]); }
  if (WIN.[P].zmax[p] != 1j) { xmax = real (WIN.[P].zmax[p]); }

  #
  # Check for potential errors
  #
  
  if (xmin == xmax) 
  { 
    # As good a guess as any
    xmin = xmin - 1;
    xmax = xmax + 1;
  }
  
  #
  # Finally, adjust if log-scales
  #
  
  if (find_char (WIN.[P].gridz[p], "l"))
  {
    if (xmin <= 0 || xmax <= 0) { error ("plot: cannot plot log(z<=0)"); }
    xmin = log10 (xmin);
    xmax = log10 (xmax);
  }
  
  return << xmin = xmin ; xmax = xmax >>;
};

##############################################################################
#
# Find the X and Y scales for a single matrix. (OLD)
#

xy_scales = function ( M, p, xmin, xmax, ymin, ymax )
{  
  #
  # 1st check for un-plottable data
  #
  
  if (any (any (isinf (M))))
  { error ("plot: cannot plot infs"); }
  if (any (any (isnan (M))))
  { error ("plot: cannot plot NaNs"); }
  
  if (M.nc == 1)
  {
    xmin = 1;
    xmax = M.nr;
    ymin = min (M);
    ymax = max (M);
  else
    xmin = min (M[;1]);
    xmax = max (M[;1]);
    ymin = min (min (M[;2:M.nc]));
    ymax = max (max (M[;2:M.nc]));
  }
  
  #
  # Check computed scale limits against user's
  #
  
  if (WIN.[P].xmin[p] != 1j) { xmin = real (WIN.[P].xmin[p]); }
  if (WIN.[P].xmax[p] != 1j) { xmax = real (WIN.[P].xmax[p]); }
  if (WIN.[P].ymin[p] != 1j) { ymin = real (WIN.[P].ymin[p]); }
  if (WIN.[P].ymax[p] != 1j) { ymax = real (WIN.[P].ymax[p]); }
  
  #
  # Check for potential errors
  #
  
  if (xmin == xmax) 
  { 
    xmin = xmin - 1;
    xmax = xmax + 1;
  }
  
  if (ymin == ymax)
  {
    ymin = ymin - 1;
    ymax = ymax + 1;
  }
  
  #
  # Finally, adjust if log-scales
  #
  
  if (find_char (WIN.[P].gridx[p], "l"))
  {
    if (xmin <= 0 || xmax <= 0) { error ("plot: cannot plot log <= 0"); }
    xmin = log10 (xmin);
    xmax = log10 (xmax);
  }
  if (find_char (WIN.[P].gridy[p], "l"))
  {
    if (ymin <= 0 || ymax <= 0) { error ("plot: cannot plot log <= 0"); }
    ymin = log10 (ymin);
    ymax = log10 (ymax);
  }
  
  return 1;
};

##############################################################################
#
# Find the X, Y and Z scales for a single matrix.
#

XYZ_scales = function ( X, Y, Z, p )
{
  # X - scale
  if (any (any (isinf (X))))
  { error ("cannot plot infs"); }
  if (any (any (isnan (X))))
  { error ("cannot plot NaNs"); }
  
  xmin = min (real (X));
  xmax = max (real (X));
  
  # Y - scale
  if (any (any (isinf (Y))))
  { error ("cannot plot infs"); }
  if (any (any (isnan (Y))))
  { error ("cannot plot NaNs"); }
  
  ymin = min (real (Y));
  ymax = max (real (Y));
  
  # Z - scale
  if (any (any (isinf (Y))))
  { error ("cannot plot infs"); }
  if (any (any (isnan (Y))))
  { error ("cannot plot NaNs"); }
  
  zmin = min (min (real (Z)));
  zmax = max (max (real (Z)));
  
  #
  # Check computed scale limits against user's
  #
  
  if (WIN.[P].xmin[p] != 1j) { xmin = real (WIN.[P].xmin[p]); }
  if (WIN.[P].xmax[p] != 1j) { xmax = real (WIN.[P].xmax[p]); }
  if (WIN.[P].ymin[p] != 1j) { ymin = real (WIN.[P].ymin[p]); }
  if (WIN.[P].ymax[p] != 1j) { ymax = real (WIN.[P].ymax[p]); }
  if (WIN.[P].zmin[p] != 1j) { zmin = real (WIN.[P].zmin[p]); }
  if (WIN.[P].zmax[p] != 1j) { zmax = real (WIN.[P].zmax[p]); }
  
  #
  # Check for potential errors
  #
  
  if (xmin == xmax) 
  { 
    # As good a guess as any
    xmin = xmin - 1;
    xmax = xmax + 1;
  }
  
  if (ymin == ymax) 
  { 
    # As good a guess as any
    ymin = ymin - 1;
    ymax = ymax + 1;
  }
  
  if (zmin == zmax) 
  { 
    # As good a guess as any
    zmin = zmin - 1;
    zmax = zmax + 1;
  }
  
  #
  # Finally, adjust if log-scales
  #
  
  if (find_char (WIN.[P].grid3x[p], "l"))
  {
    if (xmin <= 0 || xmax <= 0) { error ("plot: cannot plot log(x<=0)"); }
    xmin = log10 (xmin);
    xmax = log10 (xmax);
  }
  
  if (find_char (WIN.[P].grid3y[p], "l"))
  {
    if (ymin <= 0 || ymax <= 0) { error ("plot: cannot plot log(y<=0)"); }
    ymin = log10 (ymin);
    ymax = log10 (ymax);
  }
  
  if (find_char (WIN.[P].grid3z[p], "l"))
  {
    if (zmin <= 0 || zmax <= 0) { error ("plot: cannot plot log(z<=0)"); }
    zmin = log10 (zmin);
    zmax = log10 (zmax);
  }
  
  return <<xmin=xmin; xmax=xmax; ymin=ymin; ymax=ymax; zmin=zmin; zmax=zmax>>;
};

##############################################################################
#
# Find the X and Y scales for a list of matrices
#

list_scales = function ( data, key, p )
{
  once = 1;
  
  for (i in members (data))
  {
    M = real (data.[i]);
    if (class (M) != "num") { continue; }
    if (M.n == 0) { continue; }

    if (abs (key) > M.nc)
    {
      error ("plot: KEY argument > M.nc");
    }
    
    #
    # 1st check for un-plottable data
    #
    
    if (any (any (isinf (M))))
    { error ("plot: cannot plot infs"); }
    if (any (any (isnan (M))))
    { error ("plot: cannot plot NaNs"); }
    
    k = find ((1:M.nc) != abs (key));
    if (key > 0)
    {
      if (M.nc != 1)
      {
	</xmax;xmin/> = x_scales (real(M)[;key], p);
	</ymax;ymin/> = y_scales ( real(M)[;k],   p);
      else
	</xmax;xmin/> = x_scales ( (1:M.nr)', p);
	</ymax;ymin/> = y_scales ( real(M),   p);
      }
    else if (key < 0) {
      </xmax;xmin/> = x_scales ( real(M)[;k],        p);
      </ymax;ymin/> = y_scales ( real(M)[;abs(key)], p);
    else
      </xmax;xmin/> = x_scales ( (1:M.nr)', p);
      </ymax;ymin/> = y_scales ( real(M),   p);
    } }

    if (once) 
    { 
      Xmin = xmin; Xmax = xmax; Ymin = ymin; Ymax = ymax; 
      once = 0; 
    }
    if (xmin < Xmin) { Xmin = xmin; }
    if (xmax > Xmax) { Xmax = xmax; }
    if (ymin < Ymin) { Ymin = ymin; }
    if (ymax > Ymax) { Ymax = ymax; }
  }
  
  return <<xmin=Xmin; xmax=Xmax; ymin=Ymin; ymax=Ymax >>;
};

##############################################################################
#
# Find the maximum number of elements in a bin for a single 
# column matrix.
#

hist_scales = function ( data, nbin )
{
  dmin = min (real (data));
  dmax = max (real (data));
  dbin = linspace (dmin, dmax, nbin+1);
  binval = zeros (nbin, 1);
  
  for (i in 1:nbin)
  {
    binval[i] = length (find (data >= dbin[i] && data < dbin[i+1]));
  }
  
  return max (binval);
};

##############################################################################
#
# Plot the columns of a matrix (core function)
#
# Notes: This is the core function for plotting a matrix. If the
# matrix is a single column, then the matrix elements are plotted
# versus the row numbers. If it is a multi-column matrix, then
# columns 2:N are plotted versus column 1.
#
# p, K, k and l are indices for plot features.
# p: the current plot index (the plot #)
# K: usually 0. This index is used to start of the line style and
# color index (k = color index, l = line-style index). This is mostly
# used by plot_list, which may call plot_matrix repeatedly.
# k: the line color index. This value determines the line color used
# for each column of data. If K = 0, then k goes like 2:14, then
# flops back to 1:14.
# l: the line style inex. This value determines the line style used
# for each column of data - not the line-type (points, or lines). If
# K = 0, then l goes like 2:8, then flops back to 1:8.
#
##############################################################################

plot_matrix = function ( M, key, p, K, xmin, xmax, ymin, ymax, v )
{
  np = M.nr;
  
  if (M.nc == 1)
  {
    x = 1:M.nr;
    y = real (M);
    k = mod (1+K, 14) + 1;
    l = mod (1+K, 8);
    
    if (find_char (WIN.[P].gridx[p], "l"))
    { x = log10 (x); }
    if (find_char (WIN.[P].gridy[p], "l"))
    { y = log10 (y); }
    
    pgsci (WIN.[P].color[p;k]);     # Set line color.
    pgsls (WIN.[P].lstyle[p;l]);    # Set line style.
    pgslw (WIN.[P].width[p;l]);     # Set line width.
    
    if (get_style (WIN.[P].style.[p], k-1) == "line") 
    {
      pgline (M.nr, x, y);
    else if (get_style (WIN.[P].style.[p], k-1) == "point") {
      pgpt (M.nr, x, y, WIN.[P].pstyle[p;l]);
    else if (get_style (WIN.[P].style.[p], k-1) == "line-point") {
      pgline (M.nr, x, y);
      pgpt (M.nr, x, y, WIN.[P].pstyle[p;l]);
    else {
      pgline (M.nr, x, y);
    }}}}

    #
    # Now do the legend 
    #
    
    if (!any (any (WIN.[P].desc.[p] == 1j)))
    {
      # Use the default if necessary
      if (WIN.[P].desc.[p][1] == "default") 
      {
	desc = "c1";
      else if (WIN.[P].desc.[p].n >= k-1) {
	desc = WIN.[P].desc.[p][k-1];
      else
	# Not sure what to do, user has messed up.
	desc = "";
      } }
             
      v = v - (ymax-ymin)/11;
      xl = (xmax-xmin)*[10.5/12, 11/12, 11.5/12]' + xmin;
      yl = [v, v, v]' + ymin;
      
      if (get_style (WIN.[P].style.[p], k-1) == "line") 
      {
	pgline (3, xl, yl);
      else if (get_style (WIN.[P].style.[p], k-1) == "point") {
	pgpt (3, xl, yl, WIN.[P].pstyle[p;l]);
      else if (get_style (WIN.[P].style.[p], k-1) == "line-point") {
	pgline (3, xl, yl);
	pgpt (3, xl, yl, WIN.[P].pstyle[p;l]);
      } } }

      pgptxt (xl[1]-(xmax-xmin)/25, yl[3], 0, 1, desc);
      
    }

  else

    #
    # Check for large column dimension
    #
    
    if (M.nc > 3*M.nr)
    {
      printf (" Plot %i columns and %i rows, are you sure (y/n) ? "...
               , M.nc, M.nr);
      ans = getline ("stdin");
      if (ans.[1] != "y") { return -1; }
    }
    
    ki = find ((1:M.nc) != abs (key));
    for (i in ki)
    {
      if (key > 0)
      {
	x = real (M[;key]);
	y = real (M[;i]);
      else if (key < 0) {
	x = real (M[;i]);
	y = real (M[;abs(key)]);
      else
	x = (1:M.nr)';
	y = real (M[;i]);
      } }

      # Check for log scales, adjust if necessary
      if (find_char (WIN.[P].gridx[p], "l"))
      { x = log10 (x); }
      if (find_char (WIN.[P].gridy[p], "l"))
      { y = log10 (y); }
      
      k = mod (i-1 + K, 14) + 1;
      l = mod (8 + i-2 + K, 8) + 1;
      
      pgsci (WIN.[P].color[p;k]);     # Set line color.
      pgsls (WIN.[P].lstyle[p;l]);    # Set line style.
      pgslw (WIN.[P].width[p;l]);     # Set line width.
      
      if (get_style (WIN.[P].style.[p], k-1) == "line") 
      {
	pgline (np, x, y);
      else if (get_style (WIN.[P].style.[p], k-1) == "point") {
	pgpt (np, x, y, WIN.[P].pstyle[p;l]);
      else if (get_style (WIN.[P].style.[p], k-1) == "line-point") {
	pgline (np, x, y);
	pgpt (np, x, y, WIN.[P].pstyle[p;l]);
      else {
	pgline (np, x, y);
      }}}}

      #
      # Now do the legend 
      #
      
      if (!any (any (WIN.[P].desc.[p] == 1j)))
      {
	# Use the default if necessary
	if (WIN.[P].desc.[p][1] == "default") 
	{
	  desc = "c" + num2str (i);
        else if (WIN.[P].desc.[p].n >= k-1) {
	  desc = WIN.[P].desc.[p][k-1];
        else
	  # Not sure what to do, user has messed up.
	  desc = "";
        }}
             
	v = v - (ymax-ymin)/11;
	xl = (xmax-xmin)*[10.5/12, 11/12, 11.5/12]' + xmin;
	yl = [v, v, v]' + ymin;
	
	if (get_style (WIN.[P].style.[p], k-1) == "line") 
	{
	  pgline (3, xl, yl);
        else if (get_style (WIN.[P].style.[p], k-1) == "point") {
	  pgpt (3, xl, yl, WIN.[P].pstyle[p;l]);
        else if (get_style (WIN.[P].style.[p], k-1) == "line-point") {
	  pgline (3, xl, yl);
	  pgpt (3, xl, yl, WIN.[P].pstyle[p;l]);
        }}}
	
	pgptxt (xl[1]-(xmax-xmin)/25, yl[3], 0, 1, desc);
	
      }
    }
  }
  
  return k-1;
};

##############################################################################
#
# Plot all of the matrices in a list on the same plot
#

plot_list = function ( L, key, p, xmin, xmax, ymin, ymax )
{
  k = 0;
  v = ymax - ymin;
  
  #
  # Sort out the list members
  #

  sl = list_sort (L);

  # Plot the list members with numeric labels 1st.
  if (exist (sl.num))
  {
    for (i in sl.num)
    {
      M = L.[i];
      if (class (M) != "num") { continue; }
      if ((k = plot_matrix (M, key, p, k, xmin, xmax, ymin, ymax, v)) < 0) 
      { 
	return k; 
      }
      v = v - (ymax-ymin)/11;
    }
  }

  # Now plot the list members with string labels.
  if (exist (sl.char))
  {
    for (i in sl.char)
    {
      M = L.[i];
      if (class (M) != "num") { continue; }
      if ((k = plot_matrix (M, key, p, k, xmin, xmax, ymin, ymax, v)) < 0) 
      { 
	return k; 
      }
      v = v - (ymax-ymin)/11;
    }
  }
  return 1;
};

##############################################################################
#
# Check the elements of LIST.
# LIST must contain elements `x', `y',
# and `z'
#

check_3d_list = function ( LIST )
{
  #
  # Check existence and types
  #
  
  if (class (LIST) != "list") {
    error ("plot3: argument must be a list");
  }
  if (!exist (LIST.x)) {
    error ("plot3: arg must contain `x' member");
  else if (class (LIST.x) != "num") {
    error ("plot3: x must be numeric");
  } }
  if (!exist (LIST.y)) {
    error ("plot3: arg must contain `y' member");
  else if (class (LIST.y) != "num") {
    error ("plot3: y must be numeric"); 
  } }
  if (!exist (LIST.z)) {
    error ("plot3: arg must contain `z' member");
  else if (class (LIST.z) != "num") {
    error ("plot3: z must be numeric");
  } }

  #
  # Check sizes
  #
  
  if (LIST.x.n != LIST.z.nr) 
  {
    error ("plot3: x.n != z.nr");
  }
  
  if (LIST.y.n != LIST.z.nc) 
  {
    error ("plot3: y.n != z.nc");
  }
  
};

##############################################################################
#
# Create a legend in the current plot window
#
# if pobj.desc.[p] = inf()		no legend
# if pobj.desc.[p] = "default"		default ("c1", "c2", ...)
# if pobj.desc.[p] = "string"		use "string" as description
#

#
# Set the current plot legend string
#

plegend = function ( LEGEND )
{
  check_plot_object ();
  
  p = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;	# The current index
  
  if (!exist (LEGEND)) 
  {
    WIN.[P].desc.[p] = 1j;
    return P;
  }
  
  if (class (LEGEND) == "string")
  {
    WIN.[P].desc.[p] = LEGEND;
  }
  
  return P;
};

set3d = function (bx, by, h)
{
  if (!exist (bx)) { basex = 2; else basex = bx; }
  if (!exist (by)) { basey = 2; else basey = by; }
  if (!exist (h)) { height = 4; else height = h; }
};

##############################################################################
#
# Set the symbol size and scaling.
#

plssym = function ( DEF, SCALE )
{
  if (!exist (DEF)) { DEF = 0.0; }
  if (!exist (SCALE)) { SCALE = 1.0; }

  _plssym (DEF, SCALE);
};

##############################################################################
#
# Get any user-specific startup information. 
#
# The user must have a function called:
#
#     plot_startup_stuff
#
# in their RLAB2_RC0 (.rlab) file. This next statement will
# execute that function, performing any specific operations
# the user has included.
#

if (exist (plot_startup_stuff)) 
{
  plot_startup_stuff ();
}
